##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.server.reserveSigningKey`."""

    did: t.Optional[str] = None  #: The DID to reserve a key for.


class DataDict(te.TypedDict):
    did: te.NotRequired[t.Optional[str]]  #: The DID to reserve a key for.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.reserveSigningKey`."""

    signing_key: str  #: The public key for the reserved signing key, in did:key serialization.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

if t.TYPE_CHECKING:
    from atproto_client import models

ReasonType = t.Union[
    'models.ComAtprotoModerationDefs.ReasonSpam',
    'models.ComAtprotoModerationDefs.ReasonViolation',
    'models.ComAtprotoModerationDefs.ReasonMisleading',
    'models.ComAtprotoModerationDefs.ReasonSexual',
    'models.ComAtprotoModerationDefs.ReasonRude',
    'models.ComAtprotoModerationDefs.ReasonOther',
    'models.ComAtprotoModerationDefs.ReasonAppeal',
]  #: Reason type

ReasonSpam = te.Literal[
    'com.atproto.moderation.defs#reasonSpam'
]  #: Spam: frequent unwanted promotion, replies, mentions

ReasonViolation = te.Literal[
    'com.atproto.moderation.defs#reasonViolation'
]  #: Direct violation of server rules, laws, terms of service

ReasonMisleading = te.Literal[
    'com.atproto.moderation.defs#reasonMisleading'
]  #: Misleading identity, affiliation, or content

ReasonSexual = te.Literal['com.atproto.moderation.defs#reasonSexual']  #: Unwanted or mislabeled sexual content

ReasonRude = te.Literal[
    'com.atproto.moderation.defs#reasonRude'
]  #: Rude, harassing, explicit, or otherwise unwelcoming behavior

ReasonOther = te.Literal[
    'com.atproto.moderation.defs#reasonOther'
]  #: Other: reports not falling under another report category

ReasonAppeal = te.Literal[
    'com.atproto.moderation.defs#reasonAppeal'
]  #: Appeal: appeal a previously taken moderation action



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.sync.getRepo`."""

    did: str  #: The DID of the repo.
    since: t.Optional[str] = None  #: The revision ('rev') of the repo to create a diff from.


class ParamsDict(te.TypedDict):
    did: str  #: The DID of the repo.
    since: te.NotRequired[t.Optional[str]]  #: The revision ('rev') of the repo to create a diff from.


#: Response raw data type.
Response: te.TypeAlias = bytes



import typing as t
from dataclasses import dataclass

from atproto_identity.exceptions import AtprotoDataParseError

if t.TYPE_CHECKING:
    from atproto_core.did_doc import DidDocument


@dataclass
class AtprotoData:
    """Dataclass for atproto data."""

    did: str
    signing_key: t.Optional[str] = None
    handle: t.Optional[str] = None
    pds: t.Optional[str] = None

    @classmethod
    def from_did_doc(cls, did_doc: 'DidDocument') -> 'AtprotoData':
        """Create AT Protocol data from DID document.

        Args:
            did_doc: DID document.

        Returns:
            :obj:`AtprotoData`: AT Protocol data.
        """
        return AtprotoData(
            did=did_doc.id,
            signing_key=did_doc.get_did_key(),
            handle=did_doc.get_handle(),
            pds=did_doc.get_pds_endpoint(),
        )


def ensure_atproto_document(did_doc: 'DidDocument') -> AtprotoData:
    """Ensure that the DID document is an AT Protocol DID document.

    Args:
        did_doc: DID document.

    Returns:
        :obj:`AtprotoData`: AT Protocol data.

    Raises:
        :obj:`AtprotoDataParseError`: If the DID document is not an AT Protocol DID document.
    """
    atproto_data = AtprotoData.from_did_doc(did_doc)

    if atproto_data.signing_key is None:
        raise AtprotoDataParseError(f'Could not parse signing_key from doc: {did_doc}')
    if atproto_data.handle is None:
        raise AtprotoDataParseError(f'Could not parse handle from doc: {did_doc}')
    if atproto_data.pds is None:
        raise AtprotoDataParseError(f'Could not parse pds from doc: {did_doc}')

    return atproto_data


def ensure_atproto_key(did_doc: 'DidDocument') -> str:
    """Ensure that the DID document has AT Protocol signing key.

    Args:
        did_doc: DID document.

    Returns:
        :obj:`str`: AT Protocol signing key.

    Raises:
        :obj:`AtprotoDataParseError`: If the DID document does not have an AT Protocol signing key.
    """
    atproto_data = AtprotoData.from_did_doc(did_doc)

    if atproto_data.signing_key is None:
        raise AtprotoDataParseError(f'Could not parse signing_key from doc: {did_doc}')

    return atproto_data.signing_key



import typing as t

from atproto_client import models
from atproto_client.models.utils import get_model_as_dict, get_or_create

from atproto_firehose.client import AsyncFirehoseClient, FirehoseClient

if t.TYPE_CHECKING:
    from atproto_firehose.models import MessageFrame

# TODO(MarshalX): Everything here could be autogenerated from the lexicon.

_SUBSCRIBE_REPOS_MESSAGE_TYPE_TO_MODEL = {
    '#commit': models.ComAtprotoSyncSubscribeRepos.Commit,
    '#handle': models.ComAtprotoSyncSubscribeRepos.Handle,
    '#migrate': models.ComAtprotoSyncSubscribeRepos.Migrate,
    '#tombstone': models.ComAtprotoSyncSubscribeRepos.Tombstone,
    '#info': models.ComAtprotoSyncSubscribeRepos.Info,
    '#identity': models.ComAtprotoSyncSubscribeRepos.Identity,
}
_SUBSCRIBE_LABELS_MESSAGE_TYPE_TO_MODEL = {
    '#label': models.ComAtprotoLabelSubscribeLabels.Labels,
    '#info': models.ComAtprotoLabelSubscribeLabels.Info,
}

#: Subscribe Repos Message
SubscribeReposMessage = t.Union[
    models.ComAtprotoSyncSubscribeRepos.Commit,
    models.ComAtprotoSyncSubscribeRepos.Handle,
    models.ComAtprotoSyncSubscribeRepos.Migrate,
    models.ComAtprotoSyncSubscribeRepos.Tombstone,
    models.ComAtprotoSyncSubscribeRepos.Info,
    models.ComAtprotoSyncSubscribeRepos.Identity,
]
#: Subscribe Labels Message
SubscribeLabelsMessage = t.Union[
    models.ComAtprotoLabelSubscribeLabels.Labels,
    models.ComAtprotoLabelSubscribeLabels.Info,
]


def parse_subscribe_repos_message(message: 'MessageFrame') -> SubscribeReposMessage:
    """Parse Firehose repositories message to the corresponding model.

    Note:
        Use `decode_inner_cbor` only when required to increase performance.

    Args:
        message: Message frame.

    Returns:
        :obj:`SubscribeReposMessage`: Corresponding message model.
    """
    model_class = _SUBSCRIBE_REPOS_MESSAGE_TYPE_TO_MODEL[message.type]
    return get_or_create(message.body, model_class)


def parse_subscribe_labels_message(message: 'MessageFrame') -> SubscribeLabelsMessage:
    """Parse Firehose labels message to the corresponding model.

    Args:
        message: Message frame.

    Returns:
        :obj:`SubscribeLabelsMessage`: Corresponding message model.
    """
    model_class = _SUBSCRIBE_LABELS_MESSAGE_TYPE_TO_MODEL[message.type]
    return get_or_create(message.body, model_class)


class FirehoseSubscribeReposClient(FirehoseClient):
    """Firehose subscribe repos client.

    Args:
        params: Parameters model.
        base_uri: Base websocket URI. Example: `wss://bsky.social/xrpc`.
    """

    def __init__(
        self,
        params: t.Optional[t.Union[dict, 'models.ComAtprotoSyncSubscribeRepos.Params']] = None,
        base_uri: t.Optional[str] = None,
    ) -> None:
        params_model = get_or_create(params, models.ComAtprotoSyncSubscribeRepos.Params)

        params_dict = None
        if params_model:
            params_dict = get_model_as_dict(params_model)

        super().__init__(method='com.atproto.sync.subscribeRepos', base_uri=base_uri, params=params_dict)


class AsyncFirehoseSubscribeReposClient(AsyncFirehoseClient):
    """Async firehose subscribe repos client.

    Args:
        params: Parameters model.
        base_uri: Base websocket URI. Example: `wss://bsky.social/xrpc`.
    """

    def __init__(
        self,
        params: t.Optional[t.Union[dict, 'models.ComAtprotoSyncSubscribeRepos.Params']] = None,
        base_uri: t.Optional[str] = None,
    ) -> None:
        params_model = get_or_create(params, models.ComAtprotoSyncSubscribeRepos.Params)

        params_dict = None
        if params_model:
            params_dict = get_model_as_dict(params_model)

        super().__init__(method='com.atproto.sync.subscribeRepos', base_uri=base_uri, params=params_dict)


# TODO(MarshalX): SubscribeLabels doesn't work yet? HTTP 502 Error


class FirehoseSubscribeLabelsClient(FirehoseClient):
    """Firehose subscribe labels client.

    Args:
        params: Parameters model.
        base_uri: Base websocket URI. Example: `wss://bsky.social/xrpc`.
    """

    def __init__(
        self,
        params: t.Optional[t.Union[dict, 'models.ComAtprotoLabelSubscribeLabels.Params']] = None,
        base_uri: t.Optional[str] = None,
    ) -> None:
        params_model = get_or_create(params, models.ComAtprotoLabelSubscribeLabels.Params)

        params_dict = None
        if params_model:
            params_dict = get_model_as_dict(params_model)

        super().__init__(method='com.atproto.label.subscribeLabels', base_uri=base_uri, params=params_dict)


class AsyncFirehoseSubscribeLabelsClient(AsyncFirehoseClient):
    """Async firehose subscribe labels client.

    Args:
        params: Parameters model.
        base_uri: Base websocket URI. Example: `wss://bsky.social/xrpc`.
    """

    def __init__(
        self,
        params: t.Optional[t.Union[dict, 'models.ComAtprotoLabelSubscribeLabels.Params']] = None,
        base_uri: t.Optional[str] = None,
    ) -> None:
        params_model = get_or_create(params, models.ComAtprotoLabelSubscribeLabels.Params)

        params_dict = None
        if params_model:
            params_dict = get_model_as_dict(params_model)

        super().__init__(method='com.atproto.label.subscribeLabels', base_uri=base_uri, params=params_dict)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.feed.getFeedGenerators`."""

    feeds: t.List[str]  #: Feeds.


class ParamsDict(te.TypedDict):
    feeds: t.List[str]  #: Feeds.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.feed.getFeedGenerators`."""

    feeds: t.List['models.AppBskyFeedDefs.GeneratorView']  #: Feeds.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.moderation.createReport`."""

    reason_type: (
        'models.ComAtprotoModerationDefs.ReasonType'
    )  #: Indicates the broad category of violation the report is for.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    reason: t.Optional[str] = None  #: Additional context about the content and violation.


class DataDict(te.TypedDict):
    reason_type: (
        'models.ComAtprotoModerationDefs.ReasonType'
    )  #: Indicates the broad category of violation the report is for.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    reason: te.NotRequired[t.Optional[str]]  #: Additional context about the content and violation.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.moderation.createReport`."""

    created_at: str  #: Created at.
    id: int  #: Id.
    reason_type: 'models.ComAtprotoModerationDefs.ReasonType'  #: Reason type.
    reported_by: str  #: Reported by.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    reason: t.Optional[str] = Field(default=None, max_length=20000)  #: Reason.



def __get_models_to_rebuild_set() -> set:
    from types import ModuleType

    not_rebuilt_classes = {
        'BaseModel',
        'ModelBase',
        'ParamsModelBase',
        'DataModelBase',
        'ResponseModelBase',
        'UnknownDict',
        'DotDict',
        'UnknownRecord',
        'RecordModelBase',
        'BlobRef',
        'CID',
    }

    # use a set to remove duplicates
    models_to_rebuild = set()

    from atproto_client import models

    for var_name in dir(models).copy():
        var_value = getattr(models, var_name)
        if not isinstance(var_value, ModuleType):
            continue

        for name, value in vars(var_value).items():
            if name.startswith('_') or name in not_rebuilt_classes:
                continue

            if hasattr(value, 'model_rebuild'):
                models_to_rebuild.add(value)

    return models_to_rebuild


def __rebuild_all_models() -> None:
    # load models to the scope
    from atproto_client import models  # noqa
    from atproto_client.models.unknown_type import UnknownType, UnknownInputType
    from atproto_client.models.blob_ref import BlobRef
    from atproto_client.models import dot_dict
    from atproto_core.cid import CIDType

    UnknownType, UnknownInputType, CIDType, dot_dict  # noqa: B018

    BlobRef.model_rebuild()
    for __model in __get_models_to_rebuild_set():
        __model.model_rebuild()


def __on_load() -> None:
    __rebuild_all_models()


def load_models() -> None:
    __on_load()



from cryptography.exceptions import InvalidSignature
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.ec import ECDSA, EllipticCurve, EllipticCurvePublicKey
from cryptography.hazmat.primitives.asymmetric.utils import encode_dss_signature
from cryptography.hazmat.primitives.hashes import SHA256

from atproto_crypto.exceptions import InvalidCompressedPubkeyError


class AlgBase:
    """Base class for all algorithms."""

    NAME = None

    def __init__(self, curve: EllipticCurve, curve_order: int) -> None:
        self.curve = curve
        self.curve_order = curve_order

    def get_elliptic_curve_public_key(self, pubkey: bytes) -> EllipticCurvePublicKey:
        """Return the elliptic curve public key."""
        return EllipticCurvePublicKey.from_encoded_point(self.curve, pubkey)

    def compress_pubkey(self, pubkey: bytes) -> bytes:
        """Compress a public key."""
        return self.get_elliptic_curve_public_key(pubkey).public_bytes(
            encoding=serialization.Encoding.X962, format=serialization.PublicFormat.CompressedPoint
        )

    def decompress_pubkey(self, pubkey: bytes) -> bytes:
        """Decompress a public key."""
        if len(pubkey) != 33:
            raise InvalidCompressedPubkeyError('Expected 33 byte compress pubkey')

        return self.get_elliptic_curve_public_key(pubkey).public_bytes(
            encoding=serialization.Encoding.X962, format=serialization.PublicFormat.UncompressedPoint
        )

    def _ensure_dss_signature_low_s(self, s: int) -> None:
        """Ensure DSS signature is low-S.

        It prevents ECDSA signature malleability.

        More info: https://atproto.com/specs/cryptography#ecdsa-signature-malleability
        """
        if s > self.curve_order // 2:
            raise InvalidSignature('Invalid signature. Non low-S signature variant is denied.')

    def _encode_signature(self, signature: bytes) -> bytes:
        """Encode signature."""
        r = int.from_bytes(signature[:32], 'big')
        s = int.from_bytes(signature[32:], 'big')

        self._ensure_dss_signature_low_s(s)

        return encode_dss_signature(r, s)

    def verify_signature(self, pubkey: bytes, signing_input: bytes, signature: bytes) -> bool:
        """Verify signature.

        Args:
            pubkey: Public key.
            signing_input: Signing input (data).
            signature: Signature.

        Returns:
            :obj:`bool`: True if signature is valid, False otherwise.
        """
        try:
            self.get_elliptic_curve_public_key(pubkey).verify(
                signature=self._encode_signature(signature), data=signing_input, signature_algorithm=ECDSA(SHA256())
            )
            return True
        except InvalidSignature:
            return False



from cryptography.hazmat.primitives.asymmetric.ec import SECP256R1

from atproto_crypto.algs.base_alg import AlgBase
from atproto_crypto.consts import P256_CURVE_ORDER, P256_JWT_ALG


class P256(AlgBase):
    """P256 algorithm AKA ES256 AKA secp256r1."""

    NAME = P256_JWT_ALG

    def __init__(self) -> None:
        super().__init__(SECP256R1(), P256_CURVE_ORDER)



import typing as t
from abc import ABC, abstractmethod

from atproto_core.did_doc import DidDocument, is_valid_did_doc

from atproto_identity.did.atproto_data import ensure_atproto_document, ensure_atproto_key
from atproto_identity.exceptions import DidNotFoundError, PoorlyFormattedDidDocumentError

if t.TYPE_CHECKING:
    from atproto_identity.cache.base_cache import AsyncDidBaseCache, DidBaseCache
    from atproto_identity.did.atproto_data import AtprotoData

_DID_KEY_PREFIX = 'did:key:'


class _BaseResolver:
    @staticmethod
    def validate_did_doc(did: str, value: t.Dict[str, t.Any]) -> 'DidDocument':
        # it performs double parsing, but it's ok for now
        if not is_valid_did_doc(value):
            raise PoorlyFormattedDidDocumentError(f'Invalid DID document for DID {did}')

        did_doc = DidDocument.from_dict(value)
        if did_doc.id != did:
            raise PoorlyFormattedDidDocumentError(f'Invalid DID document for DID {did}')

        return did_doc


class BaseResolver(_BaseResolver, ABC):
    def __init__(self, cache: t.Optional['DidBaseCache'] = None) -> None:
        self._cache = cache

    @abstractmethod
    def resolve_without_validation(self, did: str) -> t.Optional[t.Dict[str, t.Any]]:
        raise NotImplementedError

    def resolve_no_cache(self, did: str) -> t.Optional['DidDocument']:
        """Resolve DID without cache.

        Args:
            did: DID.

        Returns:
            :obj:`DidDocument`: DID document or ``None`` if not found.
        """
        value = self.resolve_without_validation(did)
        if value is None:
            return None

        return self.validate_did_doc(did, value)

    def refresh_cache(self, did: str) -> None:
        """Refresh cached DID.

        Args:
            did: DID.
        """
        if self._cache is None:
            return

        self._cache.refresh(did, lambda: self.resolve_no_cache(did))

    def resolve(self, did: str, force_refresh: bool = False) -> t.Optional['DidDocument']:
        """Resolve DID.

        Args:
            did: DID.
            force_refresh: Force refresh cache.

        Returns:
            :obj:`DidDocument`: DID document or ``None`` if not found.
        """
        if self._cache and not force_refresh:
            cached_result = self._cache.get(did)
            if cached_result is not None and not cached_result.expired:
                if cached_result.stale:
                    self.refresh_cache(did)
                    cached_result = self._cache.get(did)

                return cached_result and cached_result.document

        did_doc = self.resolve_no_cache(did)
        if did_doc is None:
            if self._cache:
                self._cache.delete(did)

            return None

        if self._cache:
            self._cache.set(did, did_doc)

        return did_doc

    def ensure_resolve(self, did: str, force_refresh: bool = False) -> 'DidDocument':
        """Ensure DID is resolved.

        Args:
            did: DID.
            force_refresh: Force refresh cache.

        Returns:
            :obj:`DidDocument`: DID document.

        Raises:
            :obj:`DidNotFoundError`: DID not found.
        """
        did_doc = self.resolve(did, force_refresh)
        if did_doc is None:
            raise DidNotFoundError(f'Unable to resolve DID {did}')

        return did_doc

    def resolve_atproto_data(self, did: str, force_refresh: bool = False) -> 'AtprotoData':
        """Resolve AT Protocol Data.

        Args:
            did: DID.
            force_refresh: Force refresh cache.

        Returns:
            :obj:`AtprotoData`: AT Protocol data.

        Raises:
            :obj:`DidNotFoundError`: DID not find.
            :obj:`AtprotoDataParseError`: If the DID document is not an AT Protocol DID document.
        """
        did_doc = self.ensure_resolve(did, force_refresh)
        return ensure_atproto_document(did_doc)

    def resolve_atproto_key(self, did: str, force_refresh: bool = False) -> str:
        """Resolve AT Protocol signing key.

        Args:
            did: DID.
            force_refresh: Force refresh cache.

        Returns:
            :obj:`str`: AT Protocol signing key.

        Raises:
            :obj:`DidNotFoundError`: DID not find.
            :obj:`AtprotoDataParseError`: If the DID document does not have an AT Protocol signing key.
        """
        if did.startswith(_DID_KEY_PREFIX):
            return did

        did_doc = self.ensure_resolve(did, force_refresh)
        return ensure_atproto_key(did_doc)


class AsyncBaseResolver(_BaseResolver, ABC):
    def __init__(self, cache: t.Optional['AsyncDidBaseCache'] = None) -> None:
        self._cache = cache

    @abstractmethod
    async def resolve_without_validation(self, did: str) -> t.Optional[t.Dict[str, t.Any]]:
        raise NotImplementedError

    async def resolve_no_cache(self, did: str) -> t.Optional['DidDocument']:
        """Resolve DID without cache.

        Args:
            did: DID.

        Returns:
            :obj:`DidDocument`: DID document or ``None`` if not found.
        """
        value = await self.resolve_without_validation(did)
        if value is None:
            return None

        return self.validate_did_doc(did, value)

    async def refresh_cache(self, did: str) -> None:
        """Refresh cached DID.

        Args:
            did: DID.
        """
        if self._cache is None:
            return

        await self._cache.refresh(did, lambda: self.resolve_no_cache(did))

    async def resolve(self, did: str, force_refresh: bool = False) -> t.Optional['DidDocument']:
        """Resolve DID.

        Args:
            did: DID.
            force_refresh: Force refresh cache.

        Returns:
            :obj:`DidDocument`: DID document or ``None`` if not found.
        """
        if self._cache and not force_refresh:
            cached_result = await self._cache.get(did)
            if cached_result is not None and not cached_result.expired:
                if cached_result.stale:
                    await self.refresh_cache(did)
                    cached_result = await self._cache.get(did)

                return cached_result and cached_result.document

        did_doc = await self.resolve_no_cache(did)
        if did_doc is None:
            if self._cache:
                await self._cache.delete(did)

            return None

        if self._cache:
            await self._cache.set(did, did_doc)

        return did_doc

    async def ensure_resolve(self, did: str, force_refresh: bool = False) -> 'DidDocument':
        """Ensure DID is resolved.

        Args:
            did: DID.
            force_refresh: Force refresh cache.

        Returns:
            :obj:`DidDocument`: DID document.

        Raises:
            :obj:`DidNotFoundError`: DID not found.
        """
        did_doc = await self.resolve(did, force_refresh)
        if did_doc is None:
            raise DidNotFoundError(f'Unable to resolve DID {did}')

        return did_doc

    async def resolve_atproto_data(self, did: str, force_refresh: bool = False) -> 'AtprotoData':
        """Resolve AT Protocol Data.

        Args:
            did: DID.
            force_refresh: Force refresh cache.

        Returns:
            :obj:`AtprotoData`: AT Protocol data.

        Raises:
            :obj:`DidNotFoundError`: DID not find.
            :obj:`AtprotoDataParseError`: If the DID document is not an AT Protocol DID document.
        """
        did_doc = await self.ensure_resolve(did, force_refresh)
        return ensure_atproto_document(did_doc)

    async def resolve_atproto_key(self, did: str, force_refresh: bool = False) -> str:
        """Resolve AT Protocol signing key.

        Args:
            did: DID.
            force_refresh: Force refresh cache.

        Returns:
            :obj:`str`: AT Protocol signing key.

        Raises:
            :obj:`DidNotFoundError`: DID not find.
            :obj:`AtprotoDataParseError`: If the DID document does not have an AT Protocol signing key.
        """
        if did.startswith(_DID_KEY_PREFIX):
            return did

        did_doc = await self.ensure_resolve(did, force_refresh)
        return ensure_atproto_key(did_doc)



from dataclasses import dataclass
from enum import Enum
from typing import Optional, Union

from atproto_client.models.utils import get_or_create
from atproto_core.cbor import decode_dag_multi
from atproto_core.exceptions import AtProtocolError

from atproto_firehose.exceptions import FirehoseDecodingError


class FrameType(Enum):
    """Type of frame."""

    MESSAGE = 1
    ERROR = -1

    @classmethod
    def has_value(cls, value: int) -> bool:
        return value in cls._value2member_map_


@dataclass
class MessageFrameHeader:
    """Header of the message frame."""

    op: FrameType = FrameType.MESSAGE  #: Operation. For Message header is :obj:`FrameType.MESSAGE` always.
    t: Optional[str] = None  #: Type of body content.


@dataclass
class ErrorFrameHeader:
    """Header of the error frame."""

    op: FrameType = FrameType.ERROR  #: Operation. For Error header is :obj:`FrameType.ERROR` always.


#: Base frame header.
FrameHeader = Union[MessageFrameHeader, ErrorFrameHeader]


@dataclass
class ErrorFrameBody:
    """Body of error frame."""

    error: str  #: Code of the error.
    message: Optional[str] = None  #: Description of the error.


def parse_frame_header(raw_header: dict) -> FrameHeader:
    try:
        header_op = int(raw_header.get('op', 0))
        if not FrameType.has_value(header_op):
            raise FirehoseDecodingError('Invalid frame type')

        frame_type = FrameType(header_op)
        if frame_type is FrameType.MESSAGE:
            return get_or_create(raw_header, MessageFrameHeader)
        return get_or_create(raw_header, ErrorFrameHeader)
    except (ValueError, AtProtocolError) as e:
        raise FirehoseDecodingError('Invalid frame header') from e


def parse_frame(header: FrameHeader, raw_body: dict) -> Union['ErrorFrame', 'MessageFrame']:
    try:
        if isinstance(header, ErrorFrameHeader):
            body = get_or_create(raw_body, ErrorFrameBody)
            return ErrorFrame(header, body)
        if isinstance(header, MessageFrameHeader):
            return MessageFrame(header, raw_body)
        raise FirehoseDecodingError('Invalid frame type')
    except AtProtocolError as e:
        raise FirehoseDecodingError('Invalid frame body') from e


@dataclass
class Frame:
    """Firehose base frame."""

    header: Union[MessageFrameHeader, ErrorFrameHeader]  #: Header.
    body: Union[ErrorFrameBody, dict]  #: Body

    @property
    def operation(self) -> FrameType:
        """:obj:`FrameType`: Frame operation (frame type)."""
        return self.header.op

    @property
    def is_message(self) -> bool:
        """:obj:`bool`: Is frame the MessageFrame."""
        return self.operation is FrameType.MESSAGE

    @property
    def is_error(self) -> bool:
        """:obj:`bool`: Is frame the ErrorFrame."""
        return self.operation is FrameType.ERROR

    @staticmethod
    def from_bytes(data: Union[bytes, bytearray]) -> Union['MessageFrame', 'ErrorFrame']:
        """Decode frame from bytes of stream of bytes.

        Args:
            data: Bytes or stream of bytes of frame.

        Returns:
            :obj:`atproto.firehose_models.MessageFrame` or :obj:`atproto.firehose_models.ErrorFrame`

        Raises:
            :class:`atproto.exceptions.FirehoseError`: Invalid data frame.
        """
        decoded_parts = decode_dag_multi(data)
        if len(decoded_parts) > 2:
            raise FirehoseDecodingError('Too many CBOR data parts in the frame')
        if not len(decoded_parts):
            raise FirehoseDecodingError('Invalid frame without CBOR data')

        raw_header = decoded_parts[0]

        raw_body = None
        if len(decoded_parts) > 1:
            raw_body = decoded_parts[1]

        if raw_body is None:
            raise FirehoseDecodingError('Frame body not found')

        header = parse_frame_header(raw_header)
        return parse_frame(header, raw_body)


@dataclass
class MessageFrame(Frame):
    """Firehose message frame."""

    header: MessageFrameHeader  #: Header.
    body: dict  #: Body.

    @property
    def type(self) -> str:
        """:obj:`str`: Type of body."""
        return self.header.t


@dataclass
class ErrorFrame(Frame):
    """Firehose error frame."""

    header: ErrorFrameHeader  #: Header.
    body: ErrorFrameBody  #: Body.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.admin.getSubjectStatus`."""

    blob: t.Optional[str] = None  #: Blob.
    did: t.Optional[str] = None  #: Did.
    uri: t.Optional[str] = None  #: Uri.


class ParamsDict(te.TypedDict):
    blob: te.NotRequired[t.Optional[str]]  #: Blob.
    did: te.NotRequired[t.Optional[str]]  #: Did.
    uri: te.NotRequired[t.Optional[str]]  #: Uri.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.admin.getSubjectStatus`."""

    subject: te.Annotated[
        t.Union[
            'models.ComAtprotoAdminDefs.RepoRef',
            'models.ComAtprotoRepoStrongRef.Main',
            'models.ComAtprotoAdminDefs.RepoBlobRef',
        ],
        Field(discriminator='py_type'),
    ]  #: Subject.
    takedown: t.Optional['models.ComAtprotoAdminDefs.StatusAttr'] = None  #: Takedown.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.admin.queryModerationStatuses`."""

    appealed: t.Optional[bool] = None  #: Get subjects in unresolved appealed status.
    comment: t.Optional[str] = None  #: Search subjects by keyword from comments.
    cursor: t.Optional[str] = None  #: Cursor.
    exclude_tags: t.Optional[t.List[str]] = None  #: Exclude tags.
    ignore_subjects: t.Optional[t.List[str]] = None  #: Ignore subjects.
    include_muted: t.Optional[
        bool
    ] = None  #: By default, we don't include muted subjects in the results. Set this to true to include them.
    last_reviewed_by: t.Optional[str] = None  #: Get all subject statuses that were reviewed by a specific moderator.
    limit: t.Optional[int] = Field(default=50, ge=1, le=100)  #: Limit.
    reported_after: t.Optional[str] = None  #: Search subjects reported after a given timestamp.
    reported_before: t.Optional[str] = None  #: Search subjects reported before a given timestamp.
    review_state: t.Optional[str] = None  #: Specify when fetching subjects in a certain state.
    reviewed_after: t.Optional[str] = None  #: Search subjects reviewed after a given timestamp.
    reviewed_before: t.Optional[str] = None  #: Search subjects reviewed before a given timestamp.
    sort_direction: t.Optional[str] = None  #: Sort direction.
    sort_field: t.Optional[str] = None  #: Sort field.
    subject: t.Optional[str] = None  #: Subject.
    tags: t.Optional[t.List[str]] = None  #: Tags.
    takendown: t.Optional[bool] = None  #: Get subjects that were taken down.


class ParamsDict(te.TypedDict):
    appealed: te.NotRequired[t.Optional[bool]]  #: Get subjects in unresolved appealed status.
    comment: te.NotRequired[t.Optional[str]]  #: Search subjects by keyword from comments.
    cursor: te.NotRequired[t.Optional[str]]  #: Cursor.
    exclude_tags: te.NotRequired[t.Optional[t.List[str]]]  #: Exclude tags.
    ignore_subjects: te.NotRequired[t.Optional[t.List[str]]]  #: Ignore subjects.
    include_muted: te.NotRequired[
        t.Optional[bool]
    ]  #: By default, we don't include muted subjects in the results. Set this to true to include them.
    last_reviewed_by: te.NotRequired[
        t.Optional[str]
    ]  #: Get all subject statuses that were reviewed by a specific moderator.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    reported_after: te.NotRequired[t.Optional[str]]  #: Search subjects reported after a given timestamp.
    reported_before: te.NotRequired[t.Optional[str]]  #: Search subjects reported before a given timestamp.
    review_state: te.NotRequired[t.Optional[str]]  #: Specify when fetching subjects in a certain state.
    reviewed_after: te.NotRequired[t.Optional[str]]  #: Search subjects reviewed after a given timestamp.
    reviewed_before: te.NotRequired[t.Optional[str]]  #: Search subjects reviewed before a given timestamp.
    sort_direction: te.NotRequired[t.Optional[str]]  #: Sort direction.
    sort_field: te.NotRequired[t.Optional[str]]  #: Sort field.
    subject: te.NotRequired[t.Optional[str]]  #: Subject.
    tags: te.NotRequired[t.Optional[t.List[str]]]  #: Tags.
    takendown: te.NotRequired[t.Optional[bool]]  #: Get subjects that were taken down.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.admin.queryModerationStatuses`."""

    subject_statuses: t.List['models.ComAtprotoAdminDefs.SubjectStatusView']  #: Subject statuses.
    cursor: t.Optional[str] = None  #: Cursor.



import typing as t
from dataclasses import dataclass

from atproto_crypto.algs.p256 import P256
from atproto_crypto.algs.secp256k1 import Secp256k1
from atproto_crypto.consts import (
    BASE58_MULTIBASE_PREFIX,
    DID_KEY_PREFIX,
    P256_DID_PREFIX,
    P256_JWT_ALG,
    SECP256K1_DID_PREFIX,
    SECP256K1_JWT_ALG,
)
from atproto_crypto.exceptions import IncorrectDidKeyPrefixError, IncorrectMultikeyPrefixError, UnsupportedKeyTypeError
from atproto_crypto.multibase import bytes_to_multibase, multibase_to_bytes


@dataclass
class Multikey:
    jwt_alg: str
    key_bytes: bytes

    @staticmethod
    def from_str(multikey: str) -> 'Multikey':
        """Create multikey from string.

        Args:
            multikey: Multikey.

        Returns:
            :obj:`Multikey`: Multikey.
        """
        return parse_multikey(multikey)

    def to_str(self) -> str:
        """Format multikey.

        Returns:
            str: Multikey.
        """
        return format_multikey(self.jwt_alg, self.key_bytes)


def get_multikey_alg(multikey: str) -> str:
    """Get JWT alg for multikey.

    Args:
        multikey: Multikey.

    Returns:
        str: JWT alg.
    """
    if not multikey.startswith(BASE58_MULTIBASE_PREFIX):
        raise IncorrectMultikeyPrefixError(f'Incorrect prefix for multikey {multikey}')

    prefixed_bytes = multibase_to_bytes(multikey)
    if prefixed_bytes.startswith(P256_DID_PREFIX):
        return P256_JWT_ALG
    if prefixed_bytes.startswith(SECP256K1_DID_PREFIX):
        return SECP256K1_JWT_ALG

    raise UnsupportedKeyTypeError('Unsupported key type')


def parse_multikey(multikey: str) -> Multikey:
    """Parse multikey.

    Args:
        multikey: Multikey.

    Returns:
        :obj:`Multikey`: Multikey.

    Raises:
        :obj:`IncorrectMultikeyPrefixError`: Incorrect prefix for multikey.
        :obj:`UnsupportedKeyTypeError`: Unsupported key type.
    """
    if not multikey.startswith(BASE58_MULTIBASE_PREFIX):
        raise IncorrectMultikeyPrefixError(f'Incorrect prefix for multikey {multikey}')

    prefixed_bytes = multibase_to_bytes(multikey)
    if prefixed_bytes.startswith(P256_DID_PREFIX):
        jwt_alg = P256_JWT_ALG
        compressed_key_bytes = prefixed_bytes[len(P256_DID_PREFIX) :]
        key_bytes = P256().decompress_pubkey(compressed_key_bytes)
    elif prefixed_bytes.startswith(SECP256K1_DID_PREFIX):
        jwt_alg = SECP256K1_JWT_ALG
        compressed_key_bytes = prefixed_bytes[len(SECP256K1_DID_PREFIX) :]
        key_bytes = Secp256k1().decompress_pubkey(compressed_key_bytes)
    else:
        raise UnsupportedKeyTypeError('Unsupported key type')

    return Multikey(jwt_alg, key_bytes)


def format_multikey(jwt_alg: str, key: bytes) -> str:
    """Format multikey to multibase.

    Compress pubkey and encode with base58btc.

    Args:
        jwt_alg: JWT alg.
        key: Key bytes.

    Returns:
        str: Multikey in multibase.

    Raises:
        :obj:`UnsupportedKeyTypeError`: Unsupported key type.
    """
    if jwt_alg == P256_JWT_ALG:
        prefix = P256_DID_PREFIX
        compressed_key_bytes = P256().compress_pubkey(key)
    elif jwt_alg == SECP256K1_JWT_ALG:
        prefix = SECP256K1_DID_PREFIX
        compressed_key_bytes = Secp256k1().compress_pubkey(key)
    else:
        raise UnsupportedKeyTypeError('Unsupported key type')

    prefixed_bytes = prefix + compressed_key_bytes
    return bytes_to_multibase(BASE58_MULTIBASE_PREFIX, prefixed_bytes)


def parse_did_key(did_key: str) -> Multikey:
    """Parse DID key.

    Args:
        did_key: DID key.

    Returns:
        :obj:`Multikey`: Multikey.

    Raises:
        :obj:`IncorrectDidKeyPrefixError`: Incorrect prefix for DID key.
    """
    if not did_key.startswith(DID_KEY_PREFIX):
        raise IncorrectDidKeyPrefixError(f'Incorrect prefix for DID key {did_key}')

    return parse_multikey(did_key[len(DID_KEY_PREFIX) :])


def format_did_key(jwt_alg: str, key: bytes) -> str:
    """Format DID key.

    Args:
        jwt_alg: JWT alg.
        key: Key bytes.

    Returns:
        str: DID key.
    """
    return f'{DID_KEY_PREFIX}{format_multikey(jwt_alg, key)}'


def format_did_key_multikey(multikey: str) -> str:
    """Format DID key from multikey.

    Args:
        multikey: Multikey.

    Returns:
        :obj:`str`: DID key.
    """
    return f'{DID_KEY_PREFIX}{multikey}'


def get_did_key(key_type: str, key: str) -> t.Optional[str]:
    """Get DID key.

    Args:
        key_type: Key type.
        key: Key.

    Returns:
        :obj:`str`: DID key or ``None`` if a key type is not supported.
    """
    did_key = None
    if key_type == 'EcdsaSecp256r1VerificationKey2019':
        key_bytes = multibase_to_bytes(key)
        did_key = format_did_key(P256_JWT_ALG, key_bytes)
    elif key_type == 'EcdsaSecp256k1VerificationKey2019':
        key_bytes = multibase_to_bytes(key)
        did_key = format_did_key(SECP256K1_JWT_ALG, key_bytes)
    elif key_type == 'Multikey':
        # Multikey is compressed already
        did_key = format_did_key_multikey(key)

    return did_key



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.repo.describeRepo`."""

    repo: str  #: The handle or DID of the repo.


class ParamsDict(te.TypedDict):
    repo: str  #: The handle or DID of the repo.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.repo.describeRepo`."""

    collections: t.List[str]  #: List of all the collections (NSIDs) for which this repo contains at least one record.
    did: str  #: Did.
    did_doc: 'UnknownType'  #: The complete DID document for this account.
    handle: str  #: Handle.
    handle_is_correct: bool  #: Indicates if handle is currently valid (resolves bi-directionally).



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.refreshSession`."""

    access_jwt: str  #: Access jwt.
    did: str  #: Did.
    handle: str  #: Handle.
    refresh_jwt: str  #: Refresh jwt.
    did_doc: t.Optional['UnknownType'] = None  #: Did doc.



import typing as t

from .p256 import P256
from .secp256k1 import Secp256k1

_ANY_ALG_TYPE = t.Union[t.Type[P256], t.Type[Secp256k1]]

AVAILABLE_ALGORITHMS: t.List[_ANY_ALG_TYPE] = [P256, Secp256k1]
ALGORITHM_TO_CLASS: t.Dict[str, _ANY_ALG_TYPE] = {alg.NAME: alg for alg in AVAILABLE_ALGORITHMS}

__all__ = ['P256', 'Secp256k1', 'ALGORITHM_TO_CLASS']



import typing as t
from pathlib import Path

from atproto_core.nsid import NSID
from atproto_lexicon import models
from atproto_lexicon.parser import lexicon_parse_dir

if t.TYPE_CHECKING:
    from enum import Enum

LexDefs = t.Dict[
    str,
    t.Any,
]
LexDB = t.Dict[NSID, LexDefs]


class _LexiconDir:
    dir_path: t.Optional[Path]

    def __init__(self, default_path: t.Optional[Path] = None) -> None:
        self.dir_path = default_path

    def set(self, path: Path) -> None:
        self.dir_path = path

    def get(self) -> t.Optional[Path]:
        return self.dir_path


lexicon_dir = _LexiconDir()


def _filter_defs_by_type(
    defs: t.Dict[str, models.LexDefinition], def_types: t.Union[t.Set['models.LexDefinitionType'], t.Set['Enum']]
) -> LexDefs:
    return {k: v for k, v in defs.items() if v.type in def_types}


def _build_nsid_to_defs_map(
    lexicons: t.List[models.LexiconDoc], def_types: t.Union[t.Set['models.LexDefinitionType'], t.Set['Enum']]
) -> LexDB:
    result = {}

    for lexicon in lexicons:
        nsid = NSID.from_str(lexicon.id)
        defs = _filter_defs_by_type(lexicon.defs, def_types)
        if defs:
            result[nsid] = defs

    return result


BuiltParamsModels = t.Dict[
    NSID,
    t.Dict[
        str,
        t.Union[
            models.LexXrpcQuery,
            models.LexXrpcProcedure,
            models.LexSubscription,
        ],
    ],
]


def build_params_models() -> BuiltParamsModels:
    _LEX_DEF_TYPES_FOR_PARAMS = {
        models.LexDefinitionType.QUERY,
        models.LexDefinitionType.PROCEDURE,
        models.LexDefinitionType.SUBSCRIPTION,
    }
    return _build_nsid_to_defs_map(lexicon_parse_dir(lexicon_dir.get()), _LEX_DEF_TYPES_FOR_PARAMS)


BuiltDataModels = t.Dict[NSID, t.Dict[str, t.Union[models.LexXrpcProcedure]]]


def build_data_models() -> BuiltDataModels:
    _LEX_DEF_TYPES_FOR_DATA = {models.LexDefinitionType.PROCEDURE}
    return _build_nsid_to_defs_map(lexicon_parse_dir(lexicon_dir.get()), _LEX_DEF_TYPES_FOR_DATA)


BuiltResponseModels = t.Dict[NSID, t.Dict[str, t.Union[models.LexXrpcQuery, models.LexXrpcProcedure]]]


def build_response_models() -> BuiltResponseModels:
    _LEX_DEF_TYPES_FOR_RESPONSES = {models.LexDefinitionType.QUERY, models.LexDefinitionType.PROCEDURE}
    return _build_nsid_to_defs_map(lexicon_parse_dir(lexicon_dir.get()), _LEX_DEF_TYPES_FOR_RESPONSES)


BuiltDefModels = t.Dict[
    NSID, t.Dict[str, t.Union[models.LexObject, models.LexString, models.LexToken, models.LexArray]]
]


def build_def_models() -> BuiltDefModels:
    _LEX_DEF_TYPES_FOR_DEF = {
        models.LexDefinitionType.OBJECT,
        models.LexPrimitiveType.STRING,
        models.LexDefinitionType.TOKEN,
        models.LexDefinitionType.ARRAY,
    }
    return _build_nsid_to_defs_map(lexicon_parse_dir(lexicon_dir.get()), _LEX_DEF_TYPES_FOR_DEF)


BuiltRecordModels = t.Dict[NSID, t.Dict[str, t.Union[models.LexRecord]]]


def build_record_models() -> BuiltRecordModels:
    _LEX_DEF_TYPES_FOR_RECORDS = {models.LexDefinitionType.RECORD}
    return _build_nsid_to_defs_map(lexicon_parse_dir(lexicon_dir.get()), _LEX_DEF_TYPES_FOR_RECORDS)


if __name__ == '__main__':
    build_params_models()
    build_data_models()
    build_response_models()
    build_def_models()
    build_record_models()



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing_extensions as te

from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.sync.requestCrawl`."""

    hostname: str  #: Hostname of the current service (eg, PDS) that is requesting to be crawled.


class DataDict(te.TypedDict):
    hostname: str  #: Hostname of the current service (eg, PDS) that is requesting to be crawled.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Record(base.RecordModelBase):
    """Record model for :obj:`app.bsky.feed.repost`."""

    created_at: str  #: Created at.
    subject: 'models.ComAtprotoRepoStrongRef.Main'  #: Subject.

    py_type: te.Literal['app.bsky.feed.repost'] = Field(default='app.bsky.feed.repost', alias='$type', frozen=True)


class Main(Record):
    def __init_subclass__(cls, **data: t.Any) -> None:
        import warnings

        warnings.warn('Main class is deprecated. Use Record class instead.', DeprecationWarning, stacklevel=2)
        super().__init_subclass__(**data)

    def __init__(self, **data: t.Any) -> None:
        import warnings

        warnings.warn('Main class is deprecated. Use Record class instead.', DeprecationWarning, stacklevel=2)
        super().__init__(**data)


class GetRecordResponse(base.SugarResponseModelBase):
    """Get record response for :obj:`models.AppBskyFeedRepost.Record`."""

    uri: str  #: The URI of the record.
    value: 'models.AppBskyFeedRepost.Record'  #: The record.
    cid: t.Optional[str] = None  #: The CID of the record.


class ListRecordsResponse(base.SugarResponseModelBase):
    """List records response for :obj:`models.AppBskyFeedRepost.Record`."""

    records: t.Dict[str, 'models.AppBskyFeedRepost.Record']  #: Map of URIs to records.
    cursor: t.Optional[str] = None  #: Next page cursor.


class CreateRecordResponse(base.SugarResponseModelBase):
    """Create record response for :obj:`models.AppBskyFeedRepost.Record`."""

    uri: str  #: The URI of the record.
    cid: str  #: The CID of the record.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing_extensions as te
from pydantic import Field

from atproto_client.models import base


class SkeletonSearchPost(base.ModelBase):
    """Definition model for :obj:`app.bsky.unspecced.defs`."""

    uri: str  #: Uri.

    py_type: te.Literal['app.bsky.unspecced.defs#skeletonSearchPost'] = Field(
        default='app.bsky.unspecced.defs#skeletonSearchPost', alias='$type', frozen=True
    )


class SkeletonSearchActor(base.ModelBase):
    """Definition model for :obj:`app.bsky.unspecced.defs`."""

    did: str  #: Did.

    py_type: te.Literal['app.bsky.unspecced.defs#skeletonSearchActor'] = Field(
        default='app.bsky.unspecced.defs#skeletonSearchActor', alias='$type', frozen=True
    )



from datetime import datetime, timezone


class TimeMethodsMixin:
    _SERVER_TIMEZONE = timezone.utc

    def get_current_time(self) -> datetime:
        """Get current time in Server Timezone (UTC)."""
        return datetime.now(self._SERVER_TIMEZONE)

    def get_current_time_iso(self) -> str:
        """Get current time in Server Timezone (UTC) and ISO format."""
        return self.get_current_time().isoformat()

    def get_time_from_timestamp(self, timestamp: int) -> datetime:
        """Get datetime from timestamp in Server Timezone (UTC)."""
        return datetime.fromtimestamp(timestamp, tz=self._SERVER_TIMEZONE)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.admin.emitModerationEvent`."""

    created_by: str  #: Created by.
    event: te.Annotated[
        t.Union[
            'models.ComAtprotoAdminDefs.ModEventTakedown',
            'models.ComAtprotoAdminDefs.ModEventAcknowledge',
            'models.ComAtprotoAdminDefs.ModEventEscalate',
            'models.ComAtprotoAdminDefs.ModEventComment',
            'models.ComAtprotoAdminDefs.ModEventLabel',
            'models.ComAtprotoAdminDefs.ModEventReport',
            'models.ComAtprotoAdminDefs.ModEventMute',
            'models.ComAtprotoAdminDefs.ModEventReverseTakedown',
            'models.ComAtprotoAdminDefs.ModEventUnmute',
            'models.ComAtprotoAdminDefs.ModEventEmail',
            'models.ComAtprotoAdminDefs.ModEventTag',
        ],
        Field(discriminator='py_type'),
    ]  #: Event.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    subject_blob_cids: t.Optional[t.List[str]] = None  #: Subject blob cids.


class DataDict(te.TypedDict):
    created_by: str  #: Created by.
    event: te.Annotated[
        t.Union[
            'models.ComAtprotoAdminDefs.ModEventTakedown',
            'models.ComAtprotoAdminDefs.ModEventAcknowledge',
            'models.ComAtprotoAdminDefs.ModEventEscalate',
            'models.ComAtprotoAdminDefs.ModEventComment',
            'models.ComAtprotoAdminDefs.ModEventLabel',
            'models.ComAtprotoAdminDefs.ModEventReport',
            'models.ComAtprotoAdminDefs.ModEventMute',
            'models.ComAtprotoAdminDefs.ModEventReverseTakedown',
            'models.ComAtprotoAdminDefs.ModEventUnmute',
            'models.ComAtprotoAdminDefs.ModEventEmail',
            'models.ComAtprotoAdminDefs.ModEventTag',
        ],
        Field(discriminator='py_type'),
    ]  #: Event.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    subject_blob_cids: te.NotRequired[t.Optional[t.List[str]]]  #: Subject blob cids.



import typing as t
from dataclasses import dataclass
from datetime import datetime

if t.TYPE_CHECKING:
    from atproto_core.did_doc import DidDocument


@dataclass
class CachedDid:
    """Cached DID."""

    document: 'DidDocument'
    updated_at: datetime


@dataclass
class CachedDidResult:
    """Cached DID result."""

    did: str
    document: 'DidDocument'
    updated_at: datetime
    stale: bool
    expired: bool



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.graph.getRelationships`."""

    actor: str  #: Primary account requesting relationships for.
    others: t.Optional[t.List[str]] = Field(
        default=None, max_length=30
    )  #: List of 'other' accounts to be related back to the primary.


class ParamsDict(te.TypedDict):
    actor: str  #: Primary account requesting relationships for.
    others: te.NotRequired[t.Optional[t.List[str]]]  #: List of 'other' accounts to be related back to the primary.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.graph.getRelationships`."""

    relationships: t.List[
        te.Annotated[
            t.Union['models.AppBskyGraphDefs.Relationship', 'models.AppBskyGraphDefs.NotFoundActor'],
            Field(discriminator='py_type'),
        ]
    ]  #: Relationships.
    actor: t.Optional[str] = None  #: Actor.



_DISCLAIMER_LINES = [
    "# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!",
    '# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.',
    '# This file is part of Python atproto SDK. Licenced under MIT.',
]
_MAX_DISCLAIMER_LEN = max([len(s) for s in _DISCLAIMER_LINES])
DISCLAIMER = '\n'.join(_DISCLAIMER_LINES)
DISCLAIMER = f'{"#" * _MAX_DISCLAIMER_LEN}\n{DISCLAIMER}\n{"#" * _MAX_DISCLAIMER_LEN}\n\n'

PARAMS_MODEL = 'Params'
PARAMS_DICT = 'ParamsDict'
INPUT_MODEL = 'Data'
INPUT_DICT = 'DataDict'
OUTPUT_MODEL = 'Response'



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.blob_ref import BlobRef
from atproto_client.models import base


class Main(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.external`. A representation of some externally linked content (eg, a URL and 'card'), embedded in a Bluesky record (eg, a post)."""

    external: 'models.AppBskyEmbedExternal.External'  #: External.

    py_type: te.Literal['app.bsky.embed.external'] = Field(
        default='app.bsky.embed.external', alias='$type', frozen=True
    )


class External(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.external`."""

    description: str  #: Description.
    title: str  #: Title.
    uri: str  #: Uri.
    thumb: t.Optional['BlobRef'] = None  #: Thumb.

    py_type: te.Literal['app.bsky.embed.external#external'] = Field(
        default='app.bsky.embed.external#external', alias='$type', frozen=True
    )


class View(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.external`."""

    external: 'models.AppBskyEmbedExternal.ViewExternal'  #: External.

    py_type: te.Literal['app.bsky.embed.external#view'] = Field(
        default='app.bsky.embed.external#view', alias='$type', frozen=True
    )


class ViewExternal(base.ModelBase):
    """Definition model for :obj:`app.bsky.embed.external`."""

    description: str  #: Description.
    title: str  #: Title.
    uri: str  #: Uri.
    thumb: t.Optional[str] = None  #: Thumb.

    py_type: te.Literal['app.bsky.embed.external#viewExternal'] = Field(
        default='app.bsky.embed.external#viewExternal', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.server.createInviteCode`."""

    use_count: int  #: Use count.
    for_account: t.Optional[str] = None  #: For account.


class DataDict(te.TypedDict):
    use_count: int  #: Use count.
    for_account: te.NotRequired[t.Optional[str]]  #: For account.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.createInviteCode`."""

    code: str  #: Code.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.blob_ref import BlobRef
from atproto_client.models import base


class Record(base.RecordModelBase):
    """Record model for :obj:`app.bsky.actor.profile`."""

    avatar: t.Optional[
        'BlobRef'
    ] = None  #: Small image to be displayed next to posts from account. AKA, 'profile picture'.
    banner: t.Optional['BlobRef'] = None  #: Larger horizontal image to display behind profile view.
    description: t.Optional[str] = Field(default=None, max_length=2560)  #: Free-form profile description text.
    display_name: t.Optional[str] = Field(default=None, max_length=640)  #: Display name.
    labels: t.Optional[
        te.Annotated[t.Union['models.ComAtprotoLabelDefs.SelfLabels'], Field(default=None, discriminator='py_type')]
    ] = None  #: Self-label values, specific to the Bluesky application, on the overall account.

    py_type: te.Literal['app.bsky.actor.profile'] = Field(default='app.bsky.actor.profile', alias='$type', frozen=True)


class Main(Record):
    def __init_subclass__(cls, **data: t.Any) -> None:
        import warnings

        warnings.warn('Main class is deprecated. Use Record class instead.', DeprecationWarning, stacklevel=2)
        super().__init_subclass__(**data)

    def __init__(self, **data: t.Any) -> None:
        import warnings

        warnings.warn('Main class is deprecated. Use Record class instead.', DeprecationWarning, stacklevel=2)
        super().__init__(**data)


class GetRecordResponse(base.SugarResponseModelBase):
    """Get record response for :obj:`models.AppBskyActorProfile.Record`."""

    uri: str  #: The URI of the record.
    value: 'models.AppBskyActorProfile.Record'  #: The record.
    cid: t.Optional[str] = None  #: The CID of the record.


class ListRecordsResponse(base.SugarResponseModelBase):
    """List records response for :obj:`models.AppBskyActorProfile.Record`."""

    records: t.Dict[str, 'models.AppBskyActorProfile.Record']  #: Map of URIs to records.
    cursor: t.Optional[str] = None  #: Next page cursor.


class CreateRecordResponse(base.SugarResponseModelBase):
    """Create record response for :obj:`models.AppBskyActorProfile.Record`."""

    uri: str  #: The URI of the record.
    cid: str  #: The CID of the record.
